import { NextRequest, NextResponse } from 'next/server';
import fs from 'fs';
import path from 'path';

// Function to get the total number of exams available
export function getTotalExams(): number {
  try {
    const examDir = path.join(process.cwd(), 'data', 'examen');
    const files = fs.readdirSync(examDir).filter(file => file.endsWith('.ts'));
    console.log('Detected exam files:', files);
    return files.length;
  } catch (error) {
    console.error('Error getting total exams:', error);
    return 0;
  }
}

// Fonction pour traiter les caractères d'échappement dans le texte
function processEscapeCharacters(text: string): string {
  if (!text) return text;
  
  return text
    .replace(/\\n/g, '\n') // Remplacer \n par un vrai saut de ligne
    .replace(/\\t/g, '\t') // Remplacer \t par une vraie tabulation
    .replace(/\\r/g, '\r') // Remplacer \r par un vrai retour chariot
    .replace(/\\"/g, '"') // Remplacer \" par des guillemets
    .replace(/\\\\/g, '\\'); // Remplacer \\ par un seul backslash
}

// Solution optimisée pour les API routes dynamiques dans Next.js App Router
export async function GET(request: NextRequest) {
  console.log('API GET request received:', request.url);
  
  // Récupérer le paramètre examNumber directement depuis l'URL
  const url = new URL(request.url);
  const pathParts = url.pathname.split('/');
  const examNumberStr = pathParts[pathParts.length - 1];
  console.log('Requested exam number:', examNumberStr);
  
  try {
    // Utiliser le paramètre extrait de l'URL
    const examNum = parseInt(examNumberStr);
    console.log('Parsed exam number:', examNum);
    
    if (isNaN(examNum) || examNum < 1) {
      console.error('Invalid exam number');
      return NextResponse.json(
        { error: 'Invalid exam number' },
        { status: 400 }
      );
    }
    
    // Get the total number of available exams
    const totalExams = getTotalExams();
    console.log('Total exams available:', totalExams);
    
    // If the requested exam number is greater than the total available exams,
    // return a clear error message instead of defaulting to exam 1
    if (examNum > totalExams) {
      console.error(`Exam ${examNum} exceeds available exams (${totalExams})`);
      return NextResponse.json({
        error: `L'examen ${examNum} n'existe pas. Nous avons seulement ${totalExams} examens disponibles.`,
        examNumber: examNum,
        totalExams: totalExams,
        validExamRange: Array.from({ length: totalExams }, (_, i) => i + 1)
      }, { status: 404 });
    }
    
    // Use the requested exam number since it's valid
    const examToLoad = examNum;
    const examPath = path.join(process.cwd(), 'data', 'examen', `${examToLoad}.ts`);
    console.log('Loading exam from path:', examPath);
    
    // Vérifier si le fichier existe avant de le lire
    if (!fs.existsSync(examPath)) {
      console.error(`Exam file not found: ${examPath}`);
      return NextResponse.json({
        error: `Le fichier d'examen ${examToLoad} n'existe pas.`,
        examNumber: examNum,
        totalExams: getTotalExams(),
        validExamRange: Array.from({ length: getTotalExams() }, (_, i) => i + 1)
      }, { status: 404 });
    }
    
    console.log(`Reading exam file: ${examPath}`);
    // Read the file content
    const fileContent = fs.readFileSync(examPath, 'utf-8');
    console.log(`File content length: ${fileContent.length} bytes`);
    
    // Approche simplifiée pour extraire les questions
    let examQuestions = [];
    
    try {
      // Approche 1: Extraire le contenu entre "const questions = [" et "];" 
      const startMarker = 'const questions = [';
      const endMarker = '];';
      
      if (fileContent.includes(startMarker)) {
        const startIndex = fileContent.indexOf(startMarker) + startMarker.length;
        const endIndex = fileContent.indexOf(endMarker, startIndex);
        
        if (endIndex !== -1) {
          // Récupérer le contenu du tableau de questions
          const questionsArrayStr = fileContent.substring(startIndex, endIndex);
          
          // Créer une chaîne JavaScript valide pour l'évaluation
          const jsArrayStr = `[${questionsArrayStr}]`;
          
          try {
            // Évaluer la chaîne pour obtenir un tableau d'objets JavaScript
            const questionsArray = eval(jsArrayStr);
            
            // Traiter chaque question pour gérer les caractères d'échappement
            examQuestions = questionsArray.map((q: any) => ({
              ...q,
              question: processEscapeCharacters(q.question),
              explanation: processEscapeCharacters(q.explanation),
              // Traiter la suggestion selon son format
              suggestion: q.suggestion === 'input text' ? 'input text' :
                         Array.isArray(q.suggestion) ? q.suggestion :
                         typeof q.suggestion === 'string' ? [q.suggestion] :
                         ['oui', 'non']
            }));
            
            console.log(`Successfully loaded ${examQuestions.length} questions via eval`);
          } catch (evalError) {
            console.error('Error evaluating questions array:', evalError);
          }
        }
      }
      
      // Si nous n'avons pas de questions, essayer l'approche regex
      if (examQuestions.length === 0) {
        console.log('Trying regex approach');
        
        // Utiliser une expression régulière pour extraire les objets de question
        const regex = /\{qid:"([^"]+)",question:"([^"]+)",correct_answer:"([^"]+)",explanation:"([^"]*)",image_url:"([^"]*)",suggestion:([^\}]*)\}/g;
        let match;
        
        while ((match = regex.exec(fileContent)) !== null) {
          const [_, qid, question, correct_answer, explanation, image_url, suggestionStr] = match;
          
          // Traiter les caractères d'échappement dans la question
          const processedQuestion = processEscapeCharacters(question);
          const processedExplanation = processEscapeCharacters(explanation);
          
          // Traiter la suggestion selon son format
          let suggestion;
          if (suggestionStr === '"input text"') {
            suggestion = 'input text';
          } else if (suggestionStr.startsWith('[') && suggestionStr.includes(',')) {
            // C'est un tableau comme ["A","B","C"]
            suggestion = suggestionStr
              .replace(/\[|\]/g, '') // Enlever les crochets
              .split(',') // Diviser par virgule
              .map(s => s.trim().replace(/"/g, '')); // Nettoyer chaque élément
          } else if (suggestionStr.includes('"oui"') && suggestionStr.includes('"non"')) {
            // C'est un tableau ["oui","non"]
            suggestion = ['oui', 'non'];
          } else {
            // Par défaut, on met un tableau avec la valeur
            suggestion = [suggestionStr.replace(/"/g, '')];
          }
          
          // Ajouter la question formatée
          examQuestions.push({
            qid,
            question: processedQuestion,
            correct_answer,
            explanation: processedExplanation,
            image_url,
            suggestion
          });
        }
        
        console.log(`Extracted ${examQuestions.length} questions using regex`);
      }
      
      // Si nous n'avons toujours pas de questions, utiliser l'importation dynamique
      if (examQuestions.length === 0) {
        console.log('Using direct module import');
        
        try {
          // Créer un chemin relatif pour l'importation dynamique
          const modulePath = `../../../data/examen/${examToLoad}.ts`;
          
          // Importer dynamiquement le module
          const examModule = await import(modulePath);
          
          // Vérifier si le module a des questions ou un export par défaut
          let questions = [];
          if (examModule.questions) {
            questions = examModule.questions;
          } else if (examModule.default) {
            questions = examModule.default;
          }
          
          // Traiter les caractères d'échappement dans chaque question
          examQuestions = questions.map((q: any) => ({
            ...q,
            question: processEscapeCharacters(q.question),
            explanation: processEscapeCharacters(q.explanation),
            // Traiter la suggestion selon son format
            suggestion: q.suggestion === 'input text' ? 'input text' :
                       Array.isArray(q.suggestion) ? q.suggestion :
                       typeof q.suggestion === 'string' ? [q.suggestion] :
                       ['oui', 'non']
          }));
          
          console.log(`Loaded ${examQuestions.length} questions via dynamic import`);
        } catch (importError) {
          console.error('Error importing module:', importError);
        }
      }
    } catch (parseError) {
      console.error('Error parsing questions from file:', parseError);
    }
    
    // Si nous avons des questions, les renvoyer
    if (examQuestions.length > 0) {
      console.log(`Returning ${examQuestions.length} questions for exam ${examToLoad}`);
      return NextResponse.json({
        examNumber: examToLoad,
        totalExams: totalExams,
        questions: examQuestions
      });
    } else {
      // Si nous n'avons toujours pas de questions, renvoyer une erreur
      console.error(`Failed to extract questions from exam ${examToLoad}`);
      return NextResponse.json({
        error: `Impossible de charger les questions de l'examen ${examToLoad}.`,
        examNumber: examNum,
        totalExams: totalExams,
        validExamRange: Array.from({ length: totalExams }, (_, i) => i + 1)
      }, { status: 500 });
    }
  } catch (error) {
    console.error('Error loading exam:', error);
    return NextResponse.json(
      { error: 'Failed to load exam' },
      { status: 500 }
    );
  }
}
